/*
 * os.h
 *
 *  Created on: Jan 28, 2013
 *      Author: John
 */

#ifndef OS_H_
#define OS_H_

// ARM definitions
#define  ARM_MODE_ARM           0x00000000
#define  ARM_SVC_MODE_ARM      (0x00000013L + ARM_MODE_ARM)

// Program specific
#define STACKSIZE 256
#define NUMBER_OF_TASKS 6
#define NUMBER_OF_EVENTS 6

// Critical Section Functions
#define CPU_SR int

CPU_SR  CPU_SR_Save   (void);
void    CPU_SR_Restore(CPU_SR cpu_sr);

#define ENTER_CRITICAL()  {cpu_sr = CPU_SR_Save();}
#define EXIT_CRITICAL()   {CPU_SR_Restore(cpu_sr);}

// Task context management
//
// To manage the context for the tasks all we really need
// is to store the stack pointers. The actual context for the
// tasks are stored on their own stacks.
//
// Task Control Block. Use to save information about a task.
//
typedef struct _TaskBlock
{
	Int32U stackPointer;   // Task's stack pointer. Where the context is saved.
	Int32U id;
	Int32U debugId;
	Int32U delayCount;     // Delay count. Task is delayed if count > 0.
    struct _TaskBlock *next;
    struct _TaskBlock *previous;
} TaskBlock;

// Event Management
//
// Event Control Block
typedef struct _EventBlock
{
	int count;
	TaskBlock* waitListHead;       // First task in wait list
	TaskBlock* waitListTail;       // Last task in wait list
	struct _EventBlock *next;   // Next event block for use in free list
} EventBlock;

extern TaskBlock* runList;
extern TaskBlock* taskBlockFreeList;
extern EventBlock* eventBlockFreeList;

extern TaskBlock* currentTask; // pointer to current task block
extern int currentSP;          // The value of the current task's stack pointer

extern TaskBlock* IDLE_TASK;  // Pointer to the idle task

// Array of TaskBlocks. One for each task.
extern TaskBlock taskBlocks[NUMBER_OF_TASKS];

extern EventBlock eventBlocks[NUMBER_OF_EVENTS];

// OS Started is a flag to indicated if OS is ready
extern Boolean OSStarted;

void InitializeTimerTick (void);   // Initializes timer for the interrupt tick

void SwitchToCurrent(void);        // Switches to the current task
void SwitchContext(void);          // Switches context by saving old and restoring new

void OSInit(void);
void OSStart(void);

TaskBlock* OSCreateTask(int id, int* stack, void* task_address);

// Delay a task by number of ticks
void OSDelayTask(int numOfTicks);


EventBlock* OSCreateSemaphore(int count);
void OSPendSemaphore(EventBlock* eventBlock);
void OSPostSemaphore(EventBlock* eventBlock);

#endif /* OS_H_ */
