//
// First run using TinyOS with Events, then port it to uCOS.
//

// TinyOS headers
#include "lpc2478.h"
#include "init.h"
#include "print.h"
#include "os.h"


// Allocate stacks one for each task
int stackOne[STACKSIZE];
int stackTwo[STACKSIZE];
int stackThree[STACKSIZE];
int stackFour[STACKSIZE];

void TaskOne(void);   // Task one counts up from 0 and prints 10 lines
void TaskTwo(void);   // Task two counts down from 0xFFFFFFFF and prints 10 lines
void TaskThree(void); // Prints 10 lines
void TaskFour(void);  // Prints its name

// Semaphore shared by the tasks
EventBlock* sem;

///////////////////////////////////////////////////////////////////////////////////////////
//
//  main
//
//  Creates three tasks with CreateTask which initialize their TaskBlocks and stacks.
//  Starts the timer for the interrupt ticks which cause the tasks to switch.
//  Calls StartOS with sets the current task and switches to it.
//
//  This function never runs passed the call to StartOS() so it never exits.
//
int main(void)
{
    InitHardware();

    printString("University of Washington - Context_Switcher Test Application \n");

    OSInit();

    sem = OSCreateSemaphore(1);

    // Create tasks
    OSCreateTask(1, stackOne, (void*)TaskOne);
    OSCreateTask(2, stackTwo, (void*)TaskTwo);
    OSCreateTask(3, stackThree, (void*)TaskThree);

    // Create TaskFour after getting the output of the other
    // three tasks serialized. Notice what happens when it runs.

    // OSCreateTask(4, stackFour, (void*)TaskFour);

    InitializeTimerTick();

    OSStart();

    // Never gets here.
    printString("Main completed.\n");
    while(1);
    return 0;
}


