#include "lpc2478.h"
#include "sys.h"


#define TICKS_PER_SEC 20

void OSTickHandler(void);

void InitializeTimer0(void(*timerHandler)(void), int ticksPerSecond)
{
    int peripheral_clock_freq;
    int timer_counts;

#if 1
    // VICIntSelect
    // Set the vector 4 which is Timer0 to be an IRQ interrupt
    // Read the current value, And it with !(0x10), Write it back.
    VICINTSELECT= (VICINTSELECT) & ~(1 << VIC_TIMER0);

    // Set VICVectAddr4 with the value of the function pointer to Timer handler function
    VICVECTADDR4 = (Int32U)timerHandler;

    // VICIntEnable
    // Enable interrupt vector 4 which is timer 0.
    VICINTENABLE = 1 << VIC_TIMER0;
#endif
    // Counter Reset
    // When one, the Timer Counter and the Prescale Counter
    // are synchronously reset on the next positive edge of
    // PCLK. The counters remain reset until TCR is returned to zero.
    T0TCR = 1 << 1;

    // Re-enable counter by clearing reset bit
    T0TCR = 0;

    // Prescale counter - T0PC
    // The 32 bit PC is a counter which is incremented to the value stored in PR (Prescale Register).
    // When the value in PR is reached, the TC is incremented and the PC is cleared.
    // The PC is observable and controllable through the bus interface.
    // Set prescale counter to zero so that there's no division, ie, one to one ticks.
    T0PC = 0;

    // Get the peripheral clock frequency
    peripheral_clock_freq = SYS_GetFpclk(TIMER0_PCLK_OFFSET);

    // Calculate number of counts for match register
    timer_counts = peripheral_clock_freq / ticksPerSecond;

    // Match Register - T0MR0
    // MR0 can be enabled through the MCR to reset the Timer Counter, TC, stop both
    // the TC and PC, and/or generate an interrupt every time MR0 matches the TC
    T0MR0 = timer_counts;

    // Match Control Register - T0MCR
    // The MCR is used to control if an interrupt is generated and
    // if the TC is reset when a Match occurs.
    // Set T0MCR to interrupt and to clear TC everytime TC matches T0MR0
    T0MCR = 3;

    // Capture Control Register - T0CCR
    // The CCR controls which edges of the capture inputs are used to load the Capture Registers
    // and whether or not an interrupt is generated when a capture takes place.
    // Set disable capture
    T0CCR = 0;

    // External Match Register -T0EMR
    // The EMR controls the external match pins MATn.0-3 (MAT0.0-3 and MAT1.0-3 respectively).
    // Set no external match output.
    T0EMR = 0;

    // Timer Control Register -T0TCR
    // Enable timer.
    T0TCR = 1;
}

void InitializeTimerTick(void)
{
	InitializeTimer0(OSTickHandler, TICKS_PER_SEC);
}

